package com.sunmi.a4printerdemo;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * Created by zhicheng.liu on 2018/4/4
 * address :liuzhicheng@sunmi.com
 * description :
 */

public class BitmapUtils {

    private static final String TAG = "BitmapUtils";

    /**
     * 保存图片到本地
     * @param path
     * @param bitmap
     * @return
     */
    public static boolean saveBitmap(String path, String name, Bitmap bitmap){
        File checkPath = new File(path);
        if (!checkPath.exists()){
            try {
                checkPath.mkdirs();
            }catch (Exception e){
                e.printStackTrace();
                return false;
            }
        }

        File file = new File(path, name);
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(file);
            bitmap.compress(Bitmap.CompressFormat.JPEG, 80, fos);  //0-100,100是不压缩
            fos.flush();
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }finally {
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return true;
    }

    /**
     * 删除文件
     * @param filePath
     */
    public static boolean deleteFile(String filePath){
        boolean flag = false;
        File file = new File(filePath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            file.delete();
            flag = true;
        }
        return flag;
    }

    /**
     * 创建一个空白图像
     */
    private static Bitmap bmp = null;
    public static Bitmap createEmptyImage(int w, int h){
        if (bmp == null)
            bmp = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
        return bmp;
    }

    /**
     * 文本转图像
     * @param text 文本内容，支持换行符
     * @param size 文字的尺寸（这仨的单位都是像素点）
     * @param bmp 原图
     */
    public static Bitmap text2Image(String text, int size, Bitmap bmp){
        try {
            Canvas canvas = new Canvas(bmp);
            canvas.drawColor(Color.WHITE);
            Paint paint = new Paint();
            paint.setTextAlign(Paint.Align.LEFT);// 若设置为center，则文本左半部分显示不全
            paint.setColor(Color.BLACK);
            paint.setAntiAlias(true);// 消除锯齿
            paint.setTextSize(size);

            ///////////////////////
            StringBuilder sb = new StringBuilder();
            char c;
            int posX = 0;
            int posY = size;
            //text中如果包含\r和\n,则需要拆分和剔除,因为这俩被打印机用作了分段符
            for (int i = 0; i < text.length(); i++) {
                c = text.charAt(i);
                if (c == '\r' || c == '\n') {
                    canvas.drawText(sb.toString(), posX, posY, paint);
                    posY += size;
                    sb.setLength(0);
                } else {
                    sb.append(c);
                }
            }
            canvas.drawText(sb.toString(), posX, posY, paint);
            /////////////////////

            canvas.save();
            canvas.restore();

            return bmp;
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }

}
